@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2005, 2008, 2009, 2010,
- *               2011
+ *               2011, 2012, 2013, 2019
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,7 +42,7 @@
  *
  */
 
-/* $Id: config.h,v 1.114 2011/07/27 08:37:42 michaels Exp $ */
+/* $Id: config.h,v 1.137.10.3 2020/11/11 16:11:50 karls Exp $ */
 
 #ifndef _CONFIG_H_
 #define _CONFIG_H_
@@ -73,19 +73,11 @@
  * Put this on a fast, low-latency fs.  Under /tmp is usually good.
  * Note that if set, $TMPDIR is prefixed to this path.
  */
-#define SOCKD_SHMEMFILE           ".sockdshmemXXXXXXXXXX"
-
-/* default client config file. */
-#if !HAVE_SOCKS_CONFIGFILE
-#define SOCKS_CONFIGFILE         "/etc/socks.conf"
-#else
-#define SOCKS_CONFIGFILE         HAVE_ALT_SOCKS_CONFIGFILE
-#endif /* !HAVE_SOCKS_CONFIGFILE */
-
+#define SOCKD_SHMEMFILE             ".sockdshmemXXXXXXXXXX"
 
 #if COVENANT
 
-/* max length of a http request.  XXX probably to small. */
+/* max length of a http request.  XXX probably too small. */
 #define MAXREQLEN              (2048)
 
 #define DEFAULT_REALMNAME          "not the Inferno Nettverk A/S realm"
@@ -98,36 +90,6 @@
 #endif /* SOCKS_SERVER */
 
 
-/*
- * server send/receive socket buffer size for network i/o using TCP.
- * A value of 0 indicates we should not set any value, but use whatever
- * the kernel chooses.
- */
-#define SOCKS_SOCKET_SNDBUF_TCP         (0)
-#define SOCKS_SOCKET_RCVBUF_TCP         (0)
-
-/*
- * server receive socket buffer size for network i/o using UDP.
- * If you expect having to handle many large udp packets, you
- * might want to increase this.
- * A value of 0 indicates we should not set any value, but use whatever
- * the kernel chooses.
- */
-#define SOCKS_SOCKET_SNDBUF_UDP         (0)
-#define SOCKS_SOCKET_RCVBUF_UDP         (1024 * 64 * 1)
-
-#if BAREFOOTD
-/*
- * Barefootd will possibly handle hundreds, even thousands, of udp clients
- * on one socket, so that socket buffer we want to be as big as possible.
- * A value of 0 indicates we should not set any value, but use whatever
- * the kernel chooses.
- */
-#define SOCKS_SOCKET_RCVBUF_UDP_CLIENTSIDE (SOCKS_SOCKET_RCVBUF_UDP * 10)
-#define SOCKS_SOCKET_SNDBUF_UDP_CLIENTSIDE (0)
-#endif /* BAREFOOTD */
-
-
 /* if profiling is enabled, directory to store profile files in. */
 #define SOCKS_PROFILEDIR         "./.prof"
 
@@ -136,13 +98,16 @@
     * stuff only related to the server.
     */
 
+/* loglevel used for alarms. */
+#define LOG_ALARM          (LOG_WARNING)
+
 /*
  * If we are compiling with libwrap support, this sets the maximum
  * line length for a libwrap line.  Should be the same or less as the
  * one libwrap uses internally, but we don't have access to that size.
  */
 #if HAVE_LIBWRAP
-#define LIBWRAPBUF         (150)
+#define LIBWRAPBUF         (200)
 #endif /* HAVE_LIBWRAP */
 
 /*
@@ -154,14 +119,14 @@
 #elif BAREFOOTD
 #define DEFAULT_PAMSERVICENAME   "barefootd"
 #else
-#define DEFAULT_PAMSERVICENAME   "httprelayd"
+#define DEFAULT_PAMSERVICENAME   "covenantd"
 #endif
 
 /*
  * Name to give as stylename when using bsdauth for rules that don't
  * set it.
  */
-#define DEFAULT_BSDAUTHSTYLENAME "" /* use ""; NULL is default value */
+#define DEFAULT_BSDAUTHSTYLENAME ""    /* use ""; NULL is the default value. */
 
 /*
  * Name to give as servicename when starting gssapi for rules that don't
@@ -171,29 +136,6 @@
 #define DEFAULT_GSSAPIKEYTAB           "FILE:/etc/sockd.keytab"
 
 
-/*
- * Name to give as filter and attribute name for ldap server
- */
-#define DEFAULT_LDAP_FILTER        "(memberuid=%s)"
-#define DEFAULT_LDAP_ATTRIBUTE     "cn"
-
-/*
- * Name to give as filter and attribute name for Active Directory server
- */
-#define DEFAULT_LDAP_FILTER_AD     "(samaccountname=%s)"
-#define DEFAULT_LDAP_ATTRIBUTE_AD  "memberof"
-
-/*
- * Name to give as ca cert file or cert db path
- */
-#define DEFAULT_LDAP_CACERTFILE    "/etc/ssl/certs/cert.pem"
-#define DEFAULT_LDAP_CERTDBPATH    "/etc/certs"
-
-/*
- * Name to give as USER when using PAM and username is not available
- * (such as when pam is used as a clientmethod).
- */
-
 #define DEFAULT_PAM_USER "rhostusr"
 
 /*
@@ -202,49 +144,16 @@
 
 #define DEFAULT_PAM_RUSER "rhostusr"
 
-/*
- * The file the server will write it's process id to.
- * Note that this file should be in a restricted directory.
- */
-#if !HAVE_SOCKD_PIDFILE
-
-#if SOCKS_SERVER
-#define SOCKD_PIDFILE            "/var/run/sockd.pid"
-#elif BAREFOOTD
-#define SOCKD_PIDFILE            "/var/run/barefootd.pid"
-#else
-#define SOCKD_PIDFILE            "/var/run/httprelayd.pid"
-#endif
-
-#else
-#define SOCKD_PIDFILE            HAVE_ALT_SOCKD_PIDFILE
-#endif /* !HAVE_SOCKD_PIDFILE */
-
 /* default port for server to listen on. */
 #define SOCKD_PORT               (1080)
 
-/* default server configfile */
-#if !HAVE_SOCKD_CONFIGFILE
-
-#if SOCKS_SERVER
-#define SOCKD_CONFIGFILE         "/etc/sockd.conf"
-#elif BAREFOOTD
-#define SOCKD_CONFIGFILE         "/etc/barefootd.conf"
-#else
-#define SOCKD_CONFIGFILE         "/etc/httprelayd.conf"
-#endif
-
-#else
-#define SOCKD_CONFIGFILE         HAVE_ALT_SOCKD_CONFIGFILE
-#endif /* !HAVE_SOCKD_CONFIGFILE */
-
-
 /*
  * Internal buffer size for network i/o.  This is the amount of
  * buffer space set aside internally by the server for each socket.
  * It is *not* the socket buffer size.
  */
 
+#ifndef SOCKD_BUFSIZE
 #if HAVE_GSSAPI
 /*
  * Warning: this size needs to be at least big enough to hold two max-size
@@ -259,6 +168,7 @@
  */
 #define SOCKD_BUFSIZE         (1024 * 64 * 1)
 #endif /* !HAVE_GSSAPI */
+#endif /* SOCKD_BUFSIZE */
 
 /* max number of clients pending to server (argument to listen()).
  * The Apache people say:
@@ -266,8 +176,10 @@
  *   as an 8-bit data type; 512 truncated to 8-bits is 0, while 511 is
  *   255 when truncated.
  */
-#define SOCKD_MAXCLIENTQUE         (511)
+#define SOCKD_MAXCLIENTQUEUE       (511)
 
+/* how long a route blacklist should last. */
+#define ROUTEBLACKLIST_SECONDS     (60 * 5)
 
 /*
  * We try to cache resolved hostnames and addresses.  The following
@@ -278,36 +190,12 @@
 #define SOCKD_HOSTCACHE            (512)
 
 /* seconds a cache entry is to be considered valid.  Don't set below 1. */
-#define SOCKD_CACHETIMEOUT         (60)
-
-/*
- * Cache for LDAP stuff.
- */
-
-/* number of entries in cache. */
-#define SOCKD_LDAPCACHE            (512)
-
-/* seconds a cache entry is to be considered valid.  Don't set below 1. */
-#define SOCKD_LDAPCACHE_TIMEOUT    (60 * 15)
+#define SOCKD_CACHETIMEOUT         (60 * 5)
 
 /* print some statistics for every SOCKD_CACHESTAT lookup.  0 to disable. */
 #define SOCKD_LDAPCACHE_STAT       (0)
 
 
-/*
- * Number of slots to try and keep available for new clients at any given time.
- * The server tries to be a little more intelligent about this, but not much.
- */
-#define SOCKD_FREESLOTS_NEGOTIATE     (4)
-#define SOCKD_FREESLOTS_REQUEST       (4)
-#define SOCKD_FREESLOTS_IO            (4)
-
-#if SOCKD_FREESLOTS_NEGOTIATE < 1
-||  SOCKD_FREESLOTS_REQUEST < 1
-||  SOCKD_FREESLOTS_IO < 1
-#error "SOCKD_FREESLOTS_* can not be less than 1"
-#endif /* SOCKD_FREESLOTS < 1 */
-
 /*
  * Dante supports one process handling N clients, where the max value for
  * 'N' is limited by your system.
@@ -319,7 +207,7 @@
  */
 
 /*
- * max number of clients each negotiate process will handle.
+ * max number of clients each negotiate process will handle at a time.
  * You can probably set this to a big number.
  * Each client will occupy one file descriptor.
  */
@@ -327,7 +215,7 @@
 #if PRERELEASE
 #define SOCKD_NEGOTIATEMAX         (2)
 #else
-#define SOCKD_NEGOTIATEMAX         (24)
+#define SOCKD_NEGOTIATEMAX         (96)
 #endif /* !PRERELEASE */
 #endif /* SOCKD_NEGOTIATEMAX */
 
@@ -344,7 +232,7 @@
 #if PRERELEASE
 #define SOCKD_IOMAX               (2)
 #else
-#define SOCKD_IOMAX               (8)
+#define SOCKD_IOMAX               (32)
 #endif /* !PRERELEASE */
 #endif /* SOCKD_IOMAX */
 
@@ -352,6 +240,90 @@
 #error "SOCKD_NEGOTIATEMAX and SOCKD_IOMAX can not be less than 1"
 #endif
 
+/*
+ * Number of slots to try and keep available for new clients at any given time.
+ * The server tries to be a little more intelligent about this, but not much.
+ */
+#define SOCKD_FREESLOTS_NEGOTIATE     (MAX(SOCKD_NEGOTIATEMAX, 8))
+#define SOCKD_FREESLOTS_REQUEST       (MAX(SOCKD_REQUESTMAX,   16))
+#define SOCKD_FREESLOTS_IO            (MAX(SOCKD_IOMAX,        32))
+
+#if SOCKD_FREESLOTS_NEGOTIATE < 1
+||  SOCKD_FREESLOTS_REQUEST < 1
+||  SOCKD_FREESLOTS_IO < 1
+#error "SOCKD_FREESLOTS_* can not be less than 1"
+#endif /* SOCKD_FREESLOTS < 1 */
+
+   /*
+    * LDAP variables.
+    */
+
+/*
+ * Cache for LDAP stuff.
+ */
+
+/* number of entries in cache. */
+#ifndef SOCKD_LDAPCACHE
+#if PRERELEASE
+#define SOCKD_LDAPCACHE            (1)
+#else
+#define SOCKD_LDAPCACHE            (512)
+#endif /* PRERELEASE */
+#endif /* SOCKD_LDAPCACHE */
+
+/*
+ * Seconds a (user) cache entry is to be considered valid.
+ *  Don't set below 1.
+ */
+#ifndef SOCKD_LDAPCACHE_TIMEOUT
+#if PRERELEASE
+#define SOCKD_LDAPCACHE_TIMEOUT    (5)
+#else
+#define SOCKD_LDAPCACHE_TIMEOUT    (60 * 15)
+#endif /* PRERELEASE */
+#endif /* SOCKD_LDAPCACHE_TIMEOUT */
+
+/*
+ * Seconds a (sid) cache entry is to be considered valid.
+ */
+#ifndef SOCKD_LDAP_SID_CACHE_TIMEOUT
+#if PRERELEASE
+#define SOCKD_LDAP_SID_CACHE_TIMEOUT (15)
+#else
+#define SOCKD_LDAP_SID_CACHE_TIMEOUT (24 * 60 * 60)
+#endif /* PRERELEASE */
+#endif /* SOCKD_LDAP_SID_CACHE_TIMEOUT */
+
+/*
+ * Name to give as filter and attribute name for ldap server
+ */
+#define DEFAULT_LDAP_FILTER        "(memberuid=%s)"
+#define DEFAULT_LDAP_ATTRIBUTE     "cn"
+
+/*
+ * Name to give as filter and attribute name for Active Directory server
+ */
+#define DEFAULT_LDAP_FILTER_AD     "(samaccountname=%s)"
+#define DEFAULT_LDAP_ATTRIBUTE_AD  "memberof"
+
+/*
+ * Name to give as filter for user to DN mapping for ldap authentication
+ */
+#define DEFAULT_LDAP_AUTH_FILTER    "(uid=%s)"
+#define DEFAULT_LDAP_AUTH_FILTER_AD "(samaccountname=%s)"
+
+/*
+ * Name to give as ca cert file or cert db path
+ */
+#define DEFAULT_LDAP_CACERTFILE    "/etc/ssl/certs/cert.pem"
+#define DEFAULT_LDAP_CERTDBPATH    "/etc/certs"
+
+/*
+ * Name to give as USER when using PAM and username is not available
+ * (such as when pam is used as a clientmethod).
+ */
+
+
 #if BAREFOOTD
 /*
  * each i/o process should attempt to handle at least this many udp clients.
